#!/usr/bin/bash

# vim:ft=sh:fdm=marker:fmr={,}

# archlabs installer library script file
# this file is not meant to be run directly
# sourcing this file in a non bash shell is not advised

# set -n

chrun() {
    arch-chroot $MNT /bin/bash -c "$1"
}

json() {
    python3 -c "import sys, json; print(json.load(sys.stdin)['$1'])"
}

src() {
    . "$1" || { printf "\nFailed to source file %s\n" "$1"; die 1; }
}

ssd() {
    local dev=$1
    dev=${dev#/dev/}
    [[ $dev =~ nvme ]] && dev=${dev%p[0-9]*} || dev=${dev%[0-9]*}
    [[ $(cat /sys/block/$dev/queue/rotational) -eq 0 ]] || return 1
}

sysid() {
    if grep -qi 'apple' /sys/class/dmi/id/sys_vendor; then
        modprobe -r -q efivars
    else
        modprobe -q efivarfs
    fi

    if [[ -d /sys/firmware/efi ]]; then
        SYS="UEFI"
        [[ $(cat /sys/firmware/efi/fw_platform_size) == 64 ]] && IS_64BIT=true
    else
        SYS="BIOS"
    fi

    readonly BT="$DIST Installer - $SYS (x86_64) - Version $VER"
    return 0
}

checks() {
    if [[ $(whoami) != "root" ]]; then
        infobox "$_ErrTitle" "$_NotRoot\n$_Exit" && die 1
    elif ! grep -qw 'lm' /proc/cpuinfo; then
        infobox "$_ErrTitle" "$_Not64Bit\n$_Exit" && die 1
    elif ! curl -s --head 'https://www.archlinux.org/mirrorlist/all/' | head -n 1 | grep -q '200'; then
        if [[ $(systemctl is-active NetworkManager) == "active" ]] && hash nmtui >/dev/null 2>&1; then
            tput civis; nmtui
        fi
        if ! curl -s --head 'https://www.archlinux.org/mirrorlist/all/' | head -n 1 | grep -q '200'; then
            infobox "$_ErrTitle" "$_NoNetwork\n"
        fi
    fi
    return 0
}

echeck() {
    # return if the last process exited normally
    (( $? == 0 )) && return 0
    local msg="\nThe command exited abnormally: $1"

    # get error message from logfile and strip any non-printable characters,
    # escape sequences, and other messy text
    local err
    err="$(sed 's/[^[:print:]]//g; s/\[[0-9\;:]*\?m//g; s/==> //g; s/] ERROR:/]\nERROR:/g' "$ERR")"
    [[ $err != "" ]] && msgbox "$_ErrTitle" "$msg\n\nWith the following error message:\n\n$err"

    msg="$([[ $err == "" ]] && printf "%s" "$msg")\n$_ErrChoice"
    if [[ -e /tmp/debug-log && $TERM == 'linux' ]]; then
        msg="$([[ $err == "" ]] && printf "%s" "$msg")\n$_ErrChoiceConsole"
        yesno "$_ErrTitle" "$msg" "Exit & Open Log" "Ignore & Continue" && { less /tmp/debug-log; die 0; }
    else
        yesno "$_ErrTitle" "$msg" "Exit & Shutdown" "Ignore & Continue" && die 'shutdown -h now'
    fi

    return 0
}

icheck() {
    [[ $(lsblk -o MOUNTPOINT) =~ $MNT && $CONFIG_DONE == true ]] && return 0

    if ! [[ $(lsblk -o MOUNTPOINT) =~ $MNT ]]; then
        msgbox "$_ErrTitle" "$_ErrNoMount"
        SELECTED=4
    else
        msgbox "$_ErrTitle" "$_ErrNoConfig"
        SELECTED=5
    fi
    return 1
}

getinput() {
    local answer
    answer="$(dialog --cr-wrap --max-input 63 --stdout --no-cancel \
        --backtitle "$BT" --title " $1 " --inputbox "$2" 0 0 "$3")"
    [[ $? != 0 || $answer == "" ]] && return 1
    printf "$answer"
}

msgbox() {
    tput civis
    dialog --cr-wrap --backtitle "$BT" --title " $1 " --msgbox "$2\n" 0 0
}

infobox() {
    local sec="$3"
    tput civis
    dialog --cr-wrap --backtitle "$BT" --title " $1 " --infobox "$2\n" 0 0
    sleep ${sec:-2}
}

yesno() {
    tput civis
    if [[ $# -eq 5 && $5 == "no" ]]; then
        # option for default no using custom labels
        dialog --cr-wrap --backtitle "$BT" --defaultno --title " $1 " \
            --yes-label "$3" --no-label "$4" --yesno "$2\n" 0 0
    elif [[ $# -eq 4 ]]; then
        # option for custom labels with standard default yes
        dialog --cr-wrap --backtitle "$BT" --title " $1 " --yes-label "$3" \
            --no-label "$4" --yesno "$2\n" 0 0
    else
        # basic yes no without custom labels and default yes
        dialog --cr-wrap --backtitle "$BT" --title " $1 " --yesno "$2\n" 0 0
    fi
}

wrap_up() {
    yesno "$_CloseInst" "$1" "$2" "$3" || return 0
    [[ $4 == 'reboot' ]] && die 'reboot' || die 0
}

die() {
    tput cnorm
    punmount
    pgrep -f "$TERM_CMD -e tail" && pkill -f "$TERM_CMD -e tail"
    [[ $1 =~ [0-9] ]] && exit $1 || $1
}

sigint() {
    printf "\n** CTRL-C caught"
    die 1
}

oneshot() {
    [[ -e /tmp/.ai_$1 || ! $(type $1) ]] && return 0
    $1 || return 1
    touch "/tmp/.ai_$1"
    return 0
}

debug() {
    set -x
    exec 3>| /tmp/debug-log
    BASH_XTRACEFD=3
    if [[ $DISPLAY && $TERM != 'linux' ]]; then
        if [[ $TERM_CMD == 'st' ]]; then
            $TERM_CMD -e tail -f /tmp/debug-log &
        else
            $TERM_CMD -e "tail -f /tmp/debug-log" &
        fi
    fi
}
